/**
 *******************************************************************************
 * @file    main.c
 *
 * @brief   Main program body.
 *
 * @author  DPM
 *
 * @version V1.0.0
 *
 * @date    2024-5-6
 *
 *******************************************************************************/

/*
    Copyright (c) 2024, DP Microelectronics Inc.

    Redistribution and use in source and binary forms, with or without modification,
are permitted provided that the following conditions are met:

    1. Redistributions of source code must retain the above copyright notice, this
       list of conditions and the following disclaimer.
    2. Redistributions in binary form must reproduce the above copyright notice,
       this list of conditions and the following disclaimer in the documentation
       and/or other materials provided with the distribution.
    3. Neither the name of the copyright holder nor the names of its contributors
       may be used to endorse or promote products derived from this software without
       specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT,
INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT
NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
OF SUCH DAMAGE.
*/

/* Files include */
#include "main.h"
#include "dpm32m0xx.h"
#include "dpm32m0xx_eval.h"
#include "systick.h"
#include <stdio.h>

/**
  * @addtogroup DPM32M08X_LibSamples
  * @{
  */

/**
  * @addtogroup DPS
  * @{
  */

/**
  * @addtogroup DPS_WDGReset
  * @{
  */

/* Private typedef --------------------------------------------------------------*/
DSP_InitTypeStruct  DSP_InitStruct;
DSP_WDGInitTypeStruct  DSP_WDGInitStruct;

/* Private define ---------------------------------------------------------------*/
#define WDGTimOut 1

/* Private macro ----------------------------------------------------------------*/

/* Private variables ------------------------------------------------------------*/
uint32_t gprDefault[] = {0, 12, 13, 0, 45, 100, 0, 4096, 1024, 0, 128, 100, 1, 4096, 1024, 0};
uint32_t sdrDefault[] = {345, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0};
uint32_t gprCurrent[16];
uint32_t sdrCurrent[32];
uint32_t instructDefault[] =
{
  0x23541012,  \
  0x59AB4678,  \
  0x60F23CDE,  \
  0x834571F2,  \
  0xB5679456,  \
  0xD61FC6FF,  \
  0x020FF762,  \
  0x0408020D,  \
  0x10120618,  \
  0x10121012,  \
  0x10121012,  \
  0x10121012,  \
  0x10121012
};
uint16_t instructCnt = 13 * 2;

/* Private functions ------------------------------------------------------------*/

/**
  * @brief  The program entry function.
  * @retval int
  */
int main(void)
{
  /* Configure the system clock */
  SystemClockInit();
  /* Configure the system tick */
  SystickInit();
  
  /* LED Init */
  EVAL_LedInit(LED1);

  /* Print Uart Init */
  EVAL_ComInit();


  /********************* Operation  *********************/
  /* Enable clock */
  RCC_Unlock();
  RCC_AHBPeriphClockCmd(RCC_AHB_PERIPH_DSP, ENABLE);
  RCC_Lock();
  
  printf(" System reset source:0x%x \r\n", RCC_GetSysResetSource());
  DelayMs(10);
  /* Write instruct buffer */
  DSP_INSWriteBuffer(0, instructDefault, instructCnt * 2);

  /* Config GPR data and SDR data */
  for(uint16_t GPRx = DSP_GPR0; GPRx <= DSP_GPR15; GPRx++)
  {
    DSP_GPRWrite((DSP_GPRxEnum)GPRx, gprDefault[GPRx]);
  }

  for(uint16_t SDRx = DSP_SDR0; SDRx <= DSP_SDR31; SDRx++)
  {
    DSP_SDRWrite((DSP_SDRxEnum)SDRx, sdrDefault[SDRx]);
  }

  /* Config DSP */
  DSP_StructInit(&DSP_InitStruct);
  DSP_InitStruct.DSP_PcStart = 0;
  DSP_InitStruct.DSP_PcEnd = instructCnt - 1;
  DSP_Init(&DSP_InitStruct);
  /* Enabel DSP */
  DSP_Cmd(ENABLE);
  /* Config WDG */
  DSP_WDGInitStruct.DSP_WDGTimOutVal = WDGTimOut;
  DSP_WDGInitStruct.DSP_WDGSystemRstEn = ENABLE;
  DSP_WDGInit(&DSP_WDGInitStruct);
  /* Enable WDG */
  DSP_WDGCmd(ENABLE);
  /* Start calculate */
  DSP_SoftTrigStart();

  while(DSP_GetIntFlagStatus(DSP_INT_FLAG_DONE) != SET) ;

  DSP_ClearIntFlag(DSP_INT_FLAG_DONE);
  
  uint32_t ledTime = 0;
  /* Infinite loop */
  while(1)
  {
    /* Runing led toggle */
    if(ledTime < GetTick())
    {
      ledTime = GetTick() + 1000;
      EVAL_LedToggle(LED1);
    }
  }
}

/* retarget the C library printf function to the usart */
int fputc(int ch, FILE *f)
{
  UART_SendData(EVAL_COM0, (uint8_t)ch);

  while(RESET == UART_GetIntFlagStatus(EVAL_COM0, UART_INT_FLAG_TXE));

  return ch;
}


/**
  * @}
  */

/**
  * @}
  */

/**
  * @}
  */

/************************ (C) COPYRIGHT DP *****END OF FILE ***************************/
